#!/usr/local/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

import click

import pip_chill


@click.command()
@click.option(
    '--no-version', is_flag=True, default=False, help='Omit version numbers.')
@click.option(
    'show_all', '--all', is_flag=True, default=False,
    help='Show all packages.')
@click.option(
    '--verbose', '-v', is_flag=True, default=False,
    help='List commented out dependencies too'
)
def main(no_version=False, show_all=False, verbose=False):
    """Console script for pip_chill"""
    distributions, dependencies = pip_chill.chill(show_all=show_all)
    for package in distributions:
        if no_version:
            print(package.get_name_without_version())
        else:
            print(package)

    if verbose:
        for package in dependencies:
            if no_version:
                print(package.get_name_without_version())
            else:
                print(package)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Lists installed packages that are not dependencies of others"""

import pip

from utils import Distribution


def chill(show_all=False):
    if show_all:
        ignored_packages = ()
    else:
        ignored_packages = {
            'pip', 'pip-chill', 'wheel', 'setuptools', 'pkg-resources'}

    # Gather all packages that are requirements and will be auto-installed.
    distributions = {}
    dependencies = {}

    for distribution in pip.get_installed_distributions():
        if distribution.key in ignored_packages:
            continue

        if distribution.key in dependencies:
            dependencies[distribution.key].version = distribution.version
        else:
            distributions[distribution.key] = \
                Distribution(distribution.key, distribution.version)

        for requirement in distribution.requires():
            if requirement.key not in ignored_packages:
                if requirement.key in dependencies:
                    dependencies[requirement.key] \
                        .required_by.add(distribution.key)
                else:
                    dependencies[requirement.key] = Distribution(
                        requirement.key,
                        required_by=(distribution.key,))

            if requirement.key in distributions:
                dependencies[requirement.key].version \
                    = distributions.pop(requirement.key).version

    return sorted(distributions.values()), sorted(dependencies.values())



# #!/usr/local/bin/python
# # -*- coding: utf-8 -*-


class Distribution:
    def __init__(self, name, version=None, required_by=None):
        self.name = name
        self.version = version
        self.required_by = set(required_by) if required_by else set()

    def get_name_without_version(self):
        if self.required_by:
            return '# {} # Installed as dependency for {}' \
                .format(self.name, ', '.join(self.required_by))
        return self.name

    def __str__(self):
        if self.required_by:
            return '# {}=={} # Installed as dependency for {}' \
                .format(self.name, self.version, ', '.join(self.required_by))
        return '{}=={}'.format(self.name, self.version)

    def __cmp__(self, other):
        if isinstance(other, Distribution):
            return self.name == other.name

        return self.name == other

    def __lt__(self, other):
        return self.name < other.name

    def __hash__(self):
        return self.name



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'click',
]

test_requirements = [
    'pip'
]

setup(
    name='pip-chill',
    version='0.1.7',
    description="Like `pip freeze` but lists only the packages that are not "
    "dependencies of installed packages.",
    long_description=readme + '\n\n' + history,
    author="Ricardo Bánffy",
    author_email='rbanffy@gmail.com',
    url='https://github.com/rbanffy/pip-chill',
    packages=[
        'pip_chill',
    ],
    package_dir={'pip_chill':
                 'pip_chill'},
    entry_points={
        'console_scripts': [
            'pip-chill=pip_chill.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='pip-chill',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: '
        'OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
